/// @addtogroup poincare
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file TimeMap.h
///
/// @author Tomasz Kapela
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _CAPD_POINCARE_TIME_MAP_H_
#define _CAPD_POINCARE_TIME_MAP_H_

#include <string>

namespace capd{
namespace poincare{

template<typename DS>
class TimeMap
{
public:
  typedef typename DS::MapType MapType;
  typedef typename DS::FunctionType FunctionType;
  typedef typename DS::MatrixType MatrixType;
  typedef typename DS::VectorType VectorType;
  typedef typename DS::ScalarType ScalarType;

  TimeMap(DS &dynamicalSystem);

  VectorType operator()(ScalarType time, VectorType v);
  VectorType operator()(ScalarType time, VectorType v, MatrixType &derivative);

  template<typename SetType>
  VectorType operator()(ScalarType time, SetType &theSet );

  const MapType& getField() const;
  int getOrder() const;
  const ScalarType& getStep() const;

  void setOrder(int newOrder);
  void setStep(const ScalarType& newStep);

  void setParameter(const std::string& name, const ScalarType& value);

protected:

// the fields of the class

  DS& dynamicalSystem; // a template dynamical system (Taylor or C2Taylor)

  int m_dim;

}; // end of template TimeMap

// -----------------------  inline definitions -------------------------------

template <typename DS>
inline TimeMap<DS>::TimeMap(DS &ds) 
  : dynamicalSystem(ds)
{
  m_dim = ds.getField().getDimension();
}

template<typename DS>
inline const typename TimeMap<DS>::MapType& TimeMap<DS>::getField() const
{
  return dynamicalSystem.getField();
}

template<typename DS>
inline int TimeMap<DS>::getOrder() const
{
  return dynamicalSystem.getOrder();
}

template<typename DS>
inline const typename TimeMap<DS>::ScalarType& TimeMap<DS>::getStep() const
{
  return dynamicalSystem.getStep();
}

template<typename DS>
inline void TimeMap<DS>::setOrder(int newOrder)
{
  dynamicalSystem.setOrder(newOrder);
}

template<typename DS>
inline void TimeMap<DS>::setStep(const ScalarType& newStep)
{
  dynamicalSystem.setStep(newStep);
}


}} // namespace capd::poincare

#endif // _CAPD_POINCARE_TIME_MAP_H_



/// @}
