/// @addtogroup simple
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file ITaylor.h
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.

#ifndef _CAPD_SIMPLE_ITAYLOR_H_ 
#define _CAPD_SIMPLE_ITAYLOR_H_ 

#include <string>
//#include "simple/Matrix.h"
#include "dynsys2/BasicTaylor.h"
#include "dynsys2/Taylor.h"
#include "interval/interval.h"
#include "map/map.h"


namespace capd{
  namespace simple{
    
/// TODO:  Matrix, Vector, Map should be also a fascade in the future
    
    typedef capd::intervals::Interval<double, capd::rounding::DoubleRounding> Interval;
    typedef Matrix<Interval, 0, 0> Matrix;
    typedef capd::map::Map<Matrix> Map;
    
    
    class ITaylor:   public C1DynSys<Matrix> {
    protected:
      Map m_field;
      capd::dynsys2::Taylor<Map> m_Taylor;
    public:
      typedef Map MapType;
      typedef MapType::FunctionType FunctionType;
      typedef MapType::MatrixType MatrixType;
      typedef MatrixType::RowVectorType VectorType;
      typedef MatrixType::ScalarType ScalarType;
      typedef Norm<VectorType, MatrixType> NormType;
      
      
      ITaylor(MapType& field, int order, const ScalarType& step)
      : m_field(field), m_Taylor(m_field, order, step) {
      }
      
      ITaylor(const std::string& field, int order, const ScalarType& step)
      : m_field(field), m_Taylor(m_field, order, step) {
      }
      
      ITaylor(const char* field, int order, const ScalarType& step)
      : m_field(field), m_Taylor(m_field, order, step) {
      }
      
      VectorType Phi(const VectorType& iv) const {
        return m_Taylor.Phi(iv);
      }
      MatrixType JacPhi(const VectorType& iv) const{
        return m_Taylor.JacPhi(iv);
      }
      VectorType Remainder(const VectorType& iv) const{
        return m_Taylor.Remainder(iv);
      }
      
      void JacRemainder(
      const VectorType &vecEnclosure,
              const MatrixType &jacEnclosure,
              VectorType &Remainder,
              MatrixType &jRemainder
      ) const{
        return m_Taylor.JacRemainder(vecEnclosure, jacEnclosure, Remainder, jRemainder);
      }
      
      VectorType enclosure(const VectorType& x, int* found) const{
        return m_Taylor.enclosure(x, found);
      }
      VectorType enclosure(const VectorType &x) const{
        return m_Taylor.enclosure(x);
      }
      MatrixType jacEnclosure(const VectorType& enc, NormType& norm) const{
        return m_Taylor.jacEnclosure(enc, norm);
      }
      
      const MapType& getField() const{
        return m_field;
      }
      virtual void setField(const MapType& field){
        m_field = field;
      }
      

      int getOrder() const{
        return m_Taylor.getOrder();
      }
      virtual void setOrder(int newOrder){
        m_Taylor.setOrder(newOrder);
      }

      const ScalarType& getStep() const{
        return m_Taylor.getStep();
      }
      void setStep(const ScalarType& newStep){
        m_Taylor.setStep(newStep);
      }
      
      VectorType *getCoeff(){
        return m_Taylor.getCoeff();
      }
      
      MatrixType *getMatrixCoeff(){
        return m_Taylor.getMatrixCoeff();
      }
          
      void setParameter(const std::string &name, const ScalarType& value){
        m_field.setParameter(name, value);
      }
      
    };
    
    
// --------------- inline definitions -----------------
    
//###########################################################//
    
  }} // namespace capd::simple

#endif // _CAPD_SIMPLE_ITAYLOR_H_ 

/// @}
