// This is OPTIONS.H. Copyright (C) 2002-2004 by Pawel Pilarczyk.
// It contains some compiler-dependent or customizable global definitions.
// This is free software. No warranty. Consult 'license.txt' for details.
// Last revision: October 9, 2004.

#ifndef _OPTIONS_H_
#define _OPTIONS_H_


//Doc//
// System-dependent definitions:
// -----------------------------
// ppDOS - if compiled under DOS (important for opening binary files)
// ppUNIX - if compiled with a Unix-style compiler (GNU C++, for example)

// Note: For system-dependent definitions it is recommended that you set
// appropriate "define"s with your compiler directly (e.g. in a make file)
// instead of modifying this file, because the code below attempts
// to determine reasonable settings if none are specified.
// Other options must be adjusted in this header file to make your programs
// recompile with the new settings.

// --------------------------------------------------

// operating system choice: Unix-like or DOS/Windows-like;
// define ppDOS for opening binary files with the "ios::binary" flag;
// define ppUNIX to activate Unix-characteristic behavior of programs
// and to suppress functions inavailable under Unix;
// define both for GNU C++ compilers for DOS/Windows
//
#if (defined (ppUNIX) || defined (ppDOS))
#elif ((defined __GNUC__) && (defined __WIN32__))
	#define ppDOS
	#define ppUNIX
#elif (defined (DJGPP) || defined (DEVCPP) || defined (MINGW))
	#define ppDOS
	#define ppUNIX
#elif (defined (_BORLANDC_) || defined (__BORLANDC__) || defined (_MSC_VER))
	#define ppDOS
#else
	#define ppUNIX
#endif

// a replacement for the "bad_alloc" exception for incompatible compilers
//
#if (defined (_MSC_VER) || (defined (__GNUC__) && \
	(((__GNUC__ < 3) || ((__GNUC__ == 3) && (__GNUC_MINOR__ < 3))))))
namespace std
{
	typedef int bad_alloc;
}
#endif

//End//
#endif // _OPTIONS_H_

